@@ -1,7 +1,7 @@
-/* $Id: conf.c,v 1.50 2007/10/05 10:35:00 manu Exp $ */
+/* $Id: conf.c,v 1.70 2011/08/17 01:06:49 manu Exp $ */
 
 /*
- * Copyright (c) 2004 Emmanuel Dreyfus
+ * Copyright (c) 2004-2010 Emmanuel Dreyfus
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID
-__RCSID("$Id: conf.c,v 1.50 2007/10/05 10:35:00 manu Exp $");
+__RCSID("$Id: conf.c,v 1.70 2011/08/17 01:06:49 manu Exp $");
 #endif
 #endif
 
@@ -61,6 +61,7 @@ __RCSID("$Id: conf.c,v 1.50 2007/10/05 10:35:00 manu Exp $");
 #include <time.h>
 #include <sys/time.h>
 
+#include "spf.h"
 #include "acl.h"
 #ifdef USE_DNSRBL
 #include "dnsrbl.h"
@@ -68,13 +69,25 @@ __RCSID("$Id: conf.c,v 1.50 2007/10/05 10:35:00 manu Exp $");
 #ifdef USE_CURL
 #include "urlcheck.h"
 #endif
-#include "autowhite.h"
+#ifdef USE_LDAP
+#include "ldapcheck.h"
+#endif
+#ifdef USE_DKIM
+#include "dkimcheck.h"
+#endif
+#ifdef USE_P0F
+#include "p0f.h"
+#endif
+#ifdef USE_SPAMD
+#include "spamd.h"
+#endif
 #include "conf.h"
 #include "sync.h"
 #include "pending.h"
 #include "dump.h"
 #include "list.h"
 #include "macro.h"
+#include "ratelimit.h"
 #include "milter-greylist.h"
 
 #ifdef USE_DMALLOC
@@ -88,6 +101,7 @@ struct conf_rec defconf;
 pthread_key_t conf_key;
 char *conffile = CONFFILE;
 int conf_cold = 1;
+int conf_specified = 0;
 
 #define CONF_LOCK pthread_mutex_lock(&conf_lock);
 #define CONF_UNLOCK pthread_mutex_unlock(&conf_lock);
@@ -95,6 +109,7 @@ static pthread_mutex_t conf_lock = PTHREAD_MUTEX_INITIALIZER;
 static struct conf_list conf_list_head;
 static pthread_cond_t conf_update_cond = PTHREAD_COND_INITIALIZER;
 static int conf_updating;
+static char *legacy_conffile = "/etc/mail/greylist.conf";
 
 void
 conf_init(void) {
@@ -170,6 +185,7 @@ conf_load_internal(timestamp)
 		if (conf_cold)
 			exit(EX_OSERR);
 	} else {
+		SET_CLOEXEC(stream);
 		TSS_SET(conf_key, newconf);
 
 		peer_clear();
@@ -179,9 +195,16 @@ conf_load_internal(timestamp)
 #endif
 #ifdef USE_CURL
 		urlcheck_clear();
+#endif
+#ifdef USE_LDAP
+		ldapcheck_clear();
+#endif
+#ifdef USE_DKIM
+		dkimcheck_clear();
 #endif
 		all_list_clear();
 		macro_clear();
+		ratelimit_clear();
 		acl_clear();
 
 		conf_in = stream;
@@ -235,8 +258,20 @@ void
 conf_load(void) {
 	struct stat st;
 
+	if (!conf_specified && strcmp(conffile, legacy_conffile) != 0) {
+		struct stat st;
+
+		if (stat(legacy_conffile, &st) == 0) {
+			mg_log(LOG_WARNING,
+			       "legacy configuration file '%s' is used "
+			       "instead of '%s'.", legacy_conffile, conffile);
+			conffile = legacy_conffile;
+		}
+	}
+
 	if (stat(conffile, &st))
 		st.st_mtime = (time_t)0;
+
 	conf_load_internal(&st.st_mtime);
 }
 
@@ -420,8 +455,11 @@ conf_defaults(c)
 	c->c_testmode = 0;
 	c->c_delay = GLDELAY;
 	c->c_autowhite_validity = AUTOWHITE_VALIDITY;
+	c->c_tarpit = TARPIT_DURATION;
+	c->c_tarpit_scope = TAP_SESSION;
 	c->c_pidfile = NULL;
 	c->c_dumpfile = DUMPFILE;
+	c->c_dumpfile_mode = -1;
 	prefix2mask4(32, &c->c_match_mask);
 #ifdef AF_INET6
 	prefix2mask6(128, &c->c_match_mask6);
@@ -431,7 +469,7 @@ conf_defaults(c)
 	c->c_syncsrcaddr = NULL;
 	c->c_syncsrcport = NULL;
 	c->c_socket = NULL;
-	c->c_socket_mode = 0;
+	c->c_socket_mode = -1;
 	c->c_user = NULL;
 	c->c_nodetach = 0;
 	c->c_report = C_ALL;
@@ -441,6 +479,16 @@ conf_defaults(c)
 	c->c_dracdb = DRACDB;
 	c->c_nodrac = 0;
 	c->c_maxpeek = 0;
-
+	c->c_logfac = LOG_MAIL;
+	c->c_domainexact = 0;
+#ifdef USE_P0F
+	c->c_p0fsock[0] = '\0';
+#endif
+#ifdef USE_SPAMD
+	c->c_spamdsock[0] = '\0';
+	c->c_spamdsocktype[0] = '\0';
+#endif
+	c->c_syncmaxqlen = SYNC_MAXQLEN;
+	(void)memset(&c->c_localaddr, 0, sizeof(c->c_localaddr));
 	return;
 }
