@@ -1,4 +1,4 @@
-/* $Id: conf.h,v 1.40 2007/02/06 14:29:55 manu Exp $ */
+/* $Id: conf.h,v 1.54 2012/02/21 05:53:43 manu Exp $ */
 
 /*
  * Copyright (c) 2004 Emmanuel Dreyfus
@@ -72,8 +72,11 @@ struct conf_rec {
 	int c_testmode;
 	int c_delay;
 	int c_autowhite_validity;
+	int c_tarpit;
+	tarpit_scope_t c_tarpit_scope;
 	char *c_pidfile;
 	char *c_dumpfile;
+	int c_dumpfile_mode;
 	struct in_addr c_match_mask;
 #ifdef AF_INET6
 	struct in6_addr c_match_mask6;
@@ -85,6 +88,7 @@ struct conf_rec {
 	char *c_syncport;
 	char *c_syncsrcaddr;
 	char *c_syncsrcport;
+	int c_syncmaxqlen;
 	int c_nodetach;
 	int c_report;
 	int c_lazyaw;
@@ -95,6 +99,7 @@ struct conf_rec {
 	int c_nodrac;
 	int c_dump_no_time_translation;
 	int c_logexpired;
+	int c_logfac;
 	char c_pidfile_storage[QSTRLEN + 1];
 	char c_dumpfile_storage[QSTRLEN + 1];
 	char c_socket_storage[QSTRLEN + 1];
@@ -105,6 +110,16 @@ struct conf_rec {
 	char c_syncsrcport_storage[NUMLEN + 1];
 	char c_dracdb_storage[QSTRLEN + 1];
 	size_t c_maxpeek;
+	int c_domainexact;
+#ifdef USE_P0F
+	char c_p0fsock[QSTRLEN + 1];
+#endif
+#ifdef USE_SPAMD
+	char c_spamdsock[QSTRLEN + 1];
+	char c_spamdsocktype[QSTRLEN + 1];
+#endif
+	struct sockaddr_storage c_localaddr;
+	int c_fixldapcheck;
 };
 
 /* c_forced flags */
@@ -125,6 +140,9 @@ struct conf_rec {
 #define C_LAZYAW	0x02000
 #define C_MATCHMASK6	0x04000
 #define C_ACLDEBUG	0x08000
+#define C_DOMAINEXACT	0x10000
+#define C_TARPIT	0x20000
+#define C_TARPIT_SCOPE	0x40000
 #define C_NOTFORCED(x) 	((conf.c_forced & (x)) == 0) 
 
 /* c_report */
@@ -139,6 +157,7 @@ extern pthread_key_t conf_key;
 #define conf (*GET_CONF())
 extern char *conffile;
 extern int conf_cold;
+extern int conf_specified;
 
 void conf_init(void);
 void conf_load(void);
