@@ -1,4 +1,4 @@
-/* $Id: urlcheck.c,v 1.29.2.1 2008/08/22 21:25:29 manu Exp $ */
+/* $Id: urlcheck.c,v 1.42 2012/09/20 08:31:49 manu Exp $ */
 
 /*
  * Copyright (c) 2006-2007 Emmanuel Dreyfus
@@ -36,7 +36,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID
-__RCSID("$Id: urlcheck.c,v 1.29.2.1 2008/08/22 21:25:29 manu Exp $");
+__RCSID("$Id: urlcheck.c,v 1.42 2012/09/20 08:31:49 manu Exp $");
 #endif
 #endif
 
@@ -59,6 +59,7 @@ __RCSID("$Id: urlcheck.c,v 1.29.2.1 2008/08/22 21:25:29 manu Exp $");
 
 #include "milter-greylist.h"
 #include "pending.h"
+#include "spf.h"
 #include "acl.h"
 #include "conf.h"
 #include "urlcheck.h"
@@ -76,8 +77,8 @@ struct urlcheck_data {
 #define BOUNDARY_LEN	4
 struct post_data {
 	struct mlfi_priv *pd_priv;
-	struct header *pd_curhdr;
-	struct body *pd_curbody;
+	struct line *pd_curhdr;
+	struct line *pd_curbody;
 	char *pd_curptr;
 	int pd_done;
 	char pd_boundary[BOUNDARY_LEN + 1];
@@ -110,12 +111,7 @@ static int find_boundary(struct mlfi_priv *, char *);
 static size_t curl_post(void *, size_t, size_t, void *);
 static int answer_parse(struct iovec *, struct acl_param *, int, 
 			struct mlfi_priv *);
-static int answer_getline(char *, char *, struct acl_param *);
 static struct urlcheck_cnx *get_cnx(struct urlcheck_entry *);
-static void urlcheck_prop_push(char *, char *, int, struct mlfi_priv *);
-static void urlcheck_prop_clear_tmp(struct mlfi_priv *);
-static void urlcheck_prop_untmp(struct mlfi_priv *);
-
 static void urlcheck_validate_pipe(struct urlcheck_data *, 
 				   struct urlcheck_entry *, 
 				   struct urlcheck_cnx *, char *);
@@ -284,58 +280,6 @@ urlcheck_clear(void)	/* acllist must be write locked */
 }
 
 
-#if 0
-static char *
-url_encode(url)
-	char *url;
-{
-	char *cp;
-	size_t len;
-	char *out;
-	char *op;
-
-	len = 0;
-	for (cp = url; *cp; cp++) {
-		if (isalnum((int)*cp) || 
-		    (*cp == '.') || 
-		    (*cp == '-') || 
-		    (*cp == '_')) {
-			len++;
-		} else {
-			len += 3;
-		}
-	}
-	len++;
-
-	if ((out = malloc(len + 1)) == NULL) {
-		mg_log(LOG_ERR, "malloc(%d) failed", 
-		    len + 1, strerror(errno));
-		exit(EX_OSERR);
-	}
-	out[0] = '\0';
-	op = out;
-
-	for (cp = url; *cp; cp++) {
-		if (isalnum((int)*cp) || 
-		    (*cp == '.') || 
-		    (*cp == '-') || 
-		    (*cp == ':') || 
-		    (*cp == '_')) {
-			*op++ = *cp;
-		} else {
-			int i;
-
-			*op = '\0';
-			(void)snprintf(op, 4, "%%%x", *cp);
-			for (i = 0; i < 4; i++)
-				op[i] = (char)toupper((int)op[i]);
-			op += 3;
-		}
-	}
-
-	return out;
-}
-#endif
 
 static int
 find_boundary(priv, boundary)
@@ -343,20 +287,19 @@ find_boundary(priv, boundary)
 	char *boundary;
 {
 	int i;
-	struct body *b;
-	struct header *h;
+	struct line *l;
 
 	for (i = 0; i < BOUNDARY_LEN; i++)
 		boundary[i] = 'a';
 	boundary[BOUNDARY_LEN] = '\0';
 
 	do {
-		TAILQ_FOREACH(h, &priv->priv_header, h_list)
-			if (strstr(h->h_line, boundary) != NULL)
+		TAILQ_FOREACH(l, &priv->priv_header, l_list)
+			if (strstr(l->l_line, boundary) != NULL)
 				goto next;
 			
-		TAILQ_FOREACH(b, &priv->priv_body, b_list)
-			if (strstr(b->b_lines, boundary) != NULL)
+		TAILQ_FOREACH(l, &priv->priv_body, l_list)
+			if (strstr(l->l_line, boundary) != NULL)
 				goto next;
 
 		return 0;
@@ -397,7 +340,7 @@ curl_post(buffer, size, nmemb, userp)
 			exit(EX_OSERR);
 		}
 		snprintf(buffer, len, post_header_templ, pd->pd_boundary);
-		pd->pd_curptr = pd->pd_curhdr->h_line;
+		pd->pd_curptr = pd->pd_curhdr->l_line;
 		goto finish;
 	}
 
@@ -412,16 +355,16 @@ curl_post(buffer, size, nmemb, userp)
 			if (len > 0)
 				memcpy(buffer, pd->pd_curptr, len);
 
-			pd->pd_curhdr = TAILQ_NEXT(pd->pd_curhdr, h_list);
+			pd->pd_curhdr = TAILQ_NEXT(pd->pd_curhdr, l_list);
 
 			/* 
 			 * If there are no more headers, we will move to
 			 * the body on next time we are called.
 			 */
 			if (pd->pd_curhdr == NULL) 
-				pd->pd_curptr = pd->pd_curbody->b_lines;
+				pd->pd_curptr = pd->pd_curbody->l_line;
 			else
-				pd->pd_curptr = pd->pd_curhdr->h_line;
+				pd->pd_curptr = pd->pd_curhdr->l_line;
 
 		} else { /* (len > size * nmemb) */
 			/* 
@@ -445,11 +388,11 @@ curl_post(buffer, size, nmemb, userp)
 
 			/* Move to the next one */
 			pd->pd_curbody = 
-			    TAILQ_NEXT(pd->pd_curbody, b_list);
+			    TAILQ_NEXT(pd->pd_curbody, l_list);
 
 			/* If it's not the last one... */
 			if (pd->pd_curbody != NULL)
-				pd->pd_curptr = pd->pd_curbody->b_lines;
+				pd->pd_curptr = pd->pd_curbody->l_line;
 			else
 				pd->pd_curptr = NULL;
 		} else {
@@ -531,7 +474,7 @@ curl_outlet(buffer, size, nmemb, userp)
 	return (size * nmemb);
 }
 
-/* Return a locked connexion */
+/* Return a locked connection */
 static struct urlcheck_cnx *
 get_cnx(ue) 
 	struct urlcheck_entry *ue;
@@ -568,7 +511,7 @@ get_cnx(ue)
 	}
 
 	/* 
-	 * Nothing was free, we have to wait for a connexion.  
+	 * Nothing was free, we have to wait for a connection.  
 	 * Use the one that was locked for the longest time
 	 */
 	if (cnx == NULL) {
@@ -583,7 +526,7 @@ get_cnx(ue)
 	}
 
 	/* 
-	 * We now have a lock on a connexion 
+	 * We now have a lock on a connection 
 	 * Record the time 
 	 */
 	cnx->uc_old = time(NULL);
@@ -591,6 +534,66 @@ get_cnx(ue)
 	return cnx;
 }
 
+char *
+urlencode(str)
+	char *str;
+{
+	char *outstr;
+	size_t origlen, len;
+	char *cp;
+	char *dp;
+
+	origlen = strlen(str) + 1;
+	len = origlen;
+	for (cp = str; *cp; cp++) {
+		if (isalnum((int)*cp))
+			continue;
+
+		switch(*cp) {
+		case '-':
+		case '_':
+		case '.':
+		case '~':
+			break;
+		default:
+			len += 2;
+			break;
+		}
+	}
+
+	if (len == origlen)
+		return NULL;
+
+	if ((outstr = malloc(len)) == NULL) {
+		mg_log(LOG_ERR, "malloc failed");
+		exit (EX_OSERR);
+	}
+
+	dp = outstr;
+	for (cp = str; *cp; cp++) {
+		if (isalnum((int)*cp)) {
+			*dp++ = *cp;
+			continue;
+		}
+
+		switch(*cp) {
+		case '-':
+		case '_':
+		case '.':
+		case '~':
+			*dp++ = *cp;
+			break;
+		default:
+			(void)sprintf(dp, "%%%02X", *cp);
+			dp += 3;
+			break;
+		}
+	}
+
+	*dp = '\0';
+
+	return outstr;
+}
 
 int
 urlcheck_validate(ad, stage, ap, priv)
@@ -606,10 +609,13 @@ urlcheck_validate(ad, stage, ap, priv)
 	struct urlcheck_data ud;
 	struct urlcheck_cnx *cnx;
 	struct timeval tv1, tv2, tv3;
+	char *(*cv)(char *);
 
 	rcpt = priv->priv_cur_rcpt;
 	ue = ad->urlcheck;
-	url = fstring_expand(priv, rcpt, ue->u_url);
+
+	cv = (ue->u_flags & U_NOENCODE) ? NULL : *urlencode;
+	url = fstring_expand(priv, rcpt, ue->u_url, cv);
 
 	if (conf.c_debug) {
 		mg_log(LOG_DEBUG, "checking \"%s\"\n", url);
@@ -806,7 +812,8 @@ urlcheck_validate_helper(ue, cnx)
 	(void)alarm(URLCHECK_HELPER_TIMEOUT);
 
 	if (read(cnx->uc_pipe_req[0], &size, sizeof(size)) != sizeof(size)) {
-		mg_log(LOG_ERR, "urlcheck helper I/O error");
+		mg_log(LOG_ERR, "urlcheck helper I/O error (%d) "
+				"size = %ld", __LINE__, size);
 		exit(EX_OSERR);
 	}
 
@@ -819,7 +826,7 @@ urlcheck_validate_helper(ue, cnx)
 	}
 
 	if (read(cnx->uc_pipe_req[0], url, size) != size) {
-		mg_log(LOG_ERR, "urlcheck helper I/O error");
+		mg_log(LOG_ERR, "urlcheck helper I/O error (%d)", __LINE__);
 		exit(EX_OSERR);
 	}
 
@@ -840,17 +847,17 @@ urlcheck_validate_helper(ue, cnx)
 	size = ud.ud_iov.iov_len;
 	if (write(cnx->uc_pipe_rep[1], 
 	    &ud.ud_error, sizeof(ud.ud_error)) != sizeof(size)) {
-		mg_log(LOG_ERR, "urlcheck helper I/O error");
+		mg_log(LOG_ERR, "urlcheck helper I/O error (%d)", __LINE__);
 		exit(EX_OSERR);
 	}
 
 	if (write(cnx->uc_pipe_rep[1], &size, sizeof(size)) != sizeof(size)) {
-		mg_log(LOG_ERR, "urlcheck helper I/O error");
+		mg_log(LOG_ERR, "urlcheck helper I/O error (%d)", __LINE__);
 		exit(EX_OSERR);
 	}
 
 	if (write(cnx->uc_pipe_rep[1], ud.ud_iov.iov_base, size) != size) {
-		mg_log(LOG_ERR, "urlcheck helper I/O error");
+		mg_log(LOG_ERR, "urlcheck helper I/O error (%d)", __LINE__);
 		exit(EX_OSERR);
 	}
 
@@ -1130,7 +1137,7 @@ answer_parse(data, ap, flags, priv)
 		    buf[idx - 1] = '\0';
 		idx++;
 
-		if (answer_getline(linep, valp, ap) == -1) {
+		if (acl_modify_by_prop(linep, valp, ap) == -1) {
 			if (!(flags & U_GETPROP))
 				mg_log(LOG_DEBUG, 
 				    "ignoring unexpected \"%s\" => \"%s\"",
@@ -1141,7 +1148,7 @@ answer_parse(data, ap, flags, priv)
 		}
 
 		if (flags & U_GETPROP)
-			urlcheck_prop_push(linep, valp, flags, priv);
+			prop_push(linep, valp, flags & U_CLEARPROP, priv);
 	}
 
 	/* 
@@ -1150,284 +1157,12 @@ answer_parse(data, ap, flags, priv)
 	 */
 	if (flags & U_GETPROP) {
 		if (retval == 0)
-			urlcheck_prop_clear_tmp(priv);
+			prop_clear(priv, UP_TMPPROP);
 		else
-			urlcheck_prop_untmp(priv);
+			prop_untmp(priv);
 	}
 
 	return retval;
 }
 
-static int
-answer_getline(key, value, ap)
-	char *key;
-	char *value;
-	struct acl_param *ap;
-{
-#ifdef URLCHECK_DEBUG
-	if (conf.c_debug)
-		mg_log(LOG_DEBUG, "urlcheck got \"%s\" => \"%s\"",
-		    key, value);
-#endif
-	if (strcasecmp(key, "milterGreylistStatus") == 0) {
-		if ((strcasecmp(value, "Ok") == 0) ||
-		    (strcasecmp(value, "TRUE") == 0))
-		goto out;
-	}
-
-	if (strcasecmp(key, "milterGreylistAction") == 0) {
-		if (strcasecmp(value, "greylist") == 0)
-			ap->ap_type = A_GREYLIST;
-		else if (strcasecmp(value, "blacklist") == 0)
-			ap->ap_type = A_BLACKLIST;
-		else if (strcasecmp(value, "whitelist") == 0)
-			ap->ap_type = A_WHITELIST;
-		else 
-			mg_log(LOG_WARNING, "ignored greylist-type \"%s\"",
-			    value);
-		goto out;
-	}
-
-	if (strcasecmp(key, "milterGreylistDelay") == 0) {
-		ap->ap_delay = humanized_atoi(value);
-		goto out;
-	}
-
-	if (strcasecmp(key, "milterGreylistAutowhite") == 0) {
-		ap->ap_autowhite = humanized_atoi(value);
-		goto out;
-	}
-
-	if (strcasecmp(key, "milterGreylistFlushAddr") == 0) {
-		ap->ap_flags |= A_FLUSHADDR;
-		goto out;
-	}
-
-	if (strcasecmp(key, "milterGreylistCode") == 0) {
-		if ((ap->ap_code = strdup(value)) == NULL) {
-			mg_log(LOG_ERR, "strdup(\"%s\") failed: %s",
-			    key, strerror(errno));
-			exit(EX_OSERR);
-		}
-		ap->ap_flags |= A_FREE_CODE;
-		goto out;
-	}
-
-	if (strcasecmp(key, "milterGreylistEcode") == 0) {
-		if ((ap->ap_ecode = strdup(value)) == NULL) {
-			mg_log(LOG_ERR, "strdup(\"%s\") failed: %s",
-			    key, strerror(errno));
-			exit(EX_OSERR);
-		}
-		ap->ap_flags |= A_FREE_ECODE;
-		goto out;
-	}
-
-	if (strcasecmp(key, "milterGreylistMsg") == 0) {
-		if ((ap->ap_msg = strdup(value)) == NULL) {
-			mg_log(LOG_ERR, "strdup(\"%s\") failed: %s",
-			    key, strerror(errno));
-			exit(EX_OSERR);
-		}
-		ap->ap_flags |= A_FREE_MSG;
-		goto out;
-	}
-
-	if (strcasecmp(key, "milterGreylistReport") == 0) {
-		if ((ap->ap_report = strdup(value)) == NULL) {
-			mg_log(LOG_ERR, "strdup(\"%s\") failed: %s",
-			    key, strerror(errno));
-			exit(EX_OSERR);
-		}
-		ap->ap_flags |= A_FREE_REPORT;
-		goto out;
-	}
-
-	if (strcasecmp(key, "milterGreylistIgnore") == 0)
-		goto out;
-
-	return -1;
-out:
-	return 0;
-}
-
-static void
-urlcheck_prop_push(linep, valp, flags, priv)
-	char *linep;
-	char *valp;
-	int flags;
-	struct mlfi_priv *priv;
-{
-	char *cp;
-	struct urlcheck_prop *up;
-
-	if ((up = malloc(sizeof(*up))) == NULL) {
-		mg_log(LOG_ERR, "malloc failed: %s", strerror(errno));
-		exit(EX_OSERR);
-	}
-
-	if ((up->up_name = strdup(linep)) == NULL) {
-		mg_log(LOG_ERR, "strup failed: %s", strerror(errno));
-		exit(EX_OSERR);
-	}
-
-	if ((up->up_value = strdup(valp)) == NULL) {
-		mg_log(LOG_ERR, "strdup failed: %s", strerror(errno));
-		exit(EX_OSERR);
-	}
-
-	/*
-	 * Convert everything to lower-case
-	 */
-	for (cp = up->up_name; *cp; cp++)
-		*cp = (char)tolower((int)*cp);
-
-	for (cp = up->up_value; *cp; cp++)
-		*cp = (char)tolower((int)*cp);
-
-	up->up_flags = UP_TMPPROP;
-	if (flags & U_CLEARPROP);
-		up->up_flags |= UP_CLEARPROP;
-
-	LIST_INSERT_HEAD(&priv->priv_prop, up, up_list);
-
-	if (conf.c_debug)
-		mg_log(LOG_DEBUG, "got prop $%s = \"%s\"", linep, valp);
-
-	return;
-}
-
-void
-urlcheck_prop_clear_all(priv)
-	struct mlfi_priv *priv;
-{
-	struct urlcheck_prop *up;
-
-	while ((up = LIST_FIRST(&priv->priv_prop)) != NULL) {
-		free(up->up_name);
-		free(up->up_value);
-		LIST_REMOVE(up, up_list);
-		free(up);
-	}
-
-	return;
-}
-
-int 
-urlcheck_prop_string_validate(ad, stage, ap, priv)
-	acl_data_t *ad;
-	acl_stage_t stage;
-	struct acl_param *ap;
-	struct mlfi_priv *priv; 
-{
-	struct urlcheck_prop *up;
-	acl_data_t *upd;
-	char *string;
-	int retval = 0;
-
-	upd = (acl_data_t *)&ad->prop->upd_data;
-	string = fstring_expand(priv, NULL, upd->string);
-
-	LIST_FOREACH(up, &priv->priv_prop, up_list) {
-		if (strcasecmp(ad->prop->upd_name, up->up_name) != 0)
-			continue;
-
-		if (conf.c_debug)
-			mg_log(LOG_DEBUG, "test $%s = \"%s\" vs \"%s\"",
-			    up->up_name, up->up_value, string);
-
-		if (strcasecmp(up->up_value, string) == 0) {
-			retval = 1;
-			break;
-		}
-	}
-
-	free(string);	
-	return retval;
-}
-
-void
-urlcheck_prop_clear(priv)
-	struct mlfi_priv *priv; 
-{
-	struct urlcheck_prop *up;
-	struct urlcheck_prop *nup;
-
-	up = LIST_FIRST(&priv->priv_prop); 
-
-	while (up != NULL) {
-		nup = LIST_NEXT(up, up_list);
-		if (up->up_flags & UP_CLEARPROP) {
-			free(up->up_name);
-			free(up->up_value);
-			LIST_REMOVE(up, up_list);
-			free(up);
-		}
-		up = nup;
-	}
-	return;
-}
-
-static void
-urlcheck_prop_clear_tmp(priv)
-	struct mlfi_priv *priv; 
-{
-	struct urlcheck_prop *up;
-	struct urlcheck_prop *nup;
-
-	up = LIST_FIRST(&priv->priv_prop); 
-
-	while (up != NULL) {
-		nup = LIST_NEXT(up, up_list);
-		if (up->up_flags & UP_TMPPROP) {
-			free(up->up_name);
-			free(up->up_value);
-			LIST_REMOVE(up, up_list);
-			free(up);
-		}
-		up = nup;
-	}
-	return;
-}
-
-static void
-urlcheck_prop_untmp(priv)
-	struct mlfi_priv *priv; 
-{
-	struct urlcheck_prop *up;
-
-	LIST_FOREACH(up, &priv->priv_prop, up_list)
-		up->up_flags &= ~UP_TMPPROP;
-	return;
-}
-
-int 
-urlcheck_prop_regex_validate(ad, stage, ap, priv)
-	acl_data_t *ad;
-	acl_stage_t stage;
-	struct acl_param *ap;
-	struct mlfi_priv *priv; 
-{
-	struct urlcheck_prop *up;
-	acl_data_t *upd;
-	int retval = 0;
-
-	upd = (acl_data_t *)&ad->prop->upd_data;
-
-	LIST_FOREACH(up, &priv->priv_prop, up_list) {
-		if (strcasecmp(ad->prop->upd_name, up->up_name) != 0)
-			continue;
-
-		if (conf.c_debug)
-			mg_log(LOG_DEBUG, "test $%s = \"%s\" vs %s",
-			    up->up_name, up->up_value, upd->regex.re_copy);
-
-		if (myregexec(priv, upd, ap, up->up_value) == 0) {
-			retval = 1;
-			break;
-		}
-	}
-
-	return retval;
-}
 #endif /* USE_CURL */
